Attribute VB_Name = "modTrustedLocation"
Option Compare Database
Option Explicit

' Constants (taken from WinReg.h)
Const HKEY_CLASSES_ROOT = &H80000000
Const HKEY_CURRENT_USER = &H80000001
Const HKEY_LOCAL_MACHINE = &H80000002
Const HKEY_USERS = &H80000003

Const REG_SZ = 1
Const REG_EXPAND_SZ = 2
Const REG_BINARY = 3
Const REG_DWORD = 4
Const REG_MULTI_SZ = 7

'================================================================================
'Purpose:           Checks if current location is trusted
'Name:              IsTrustedLocation
'Author:            Colin Riddington, Mendip Data Systems
'First Release:     2022-10-11
'URL:               https://isladogs.co.uk/trusted-location/

'Latest Version:    3.0
'Last Updated:      2025-01-22
'Main Changes:      fixed issues with subfolders & added description
'                   renamed from IsLocationTrusted - now runs from autoexec macro
'                   output changed to FormattedMsgBox (also provided)
 '===============================================================================

Function IsTrustedLocation() As Boolean

On Error GoTo Err_Handler

        IsTrustedLocation = False

        Dim strComputer As String, hDefKey As Long, strKeyPath As String, strSubKeyPath As String, strAccessVersion As String
        Dim strValueName As String, strValue As Variant, I As Integer, oReg As Object
        Dim strSubkey As Variant, arrSubKeys As Variant, arrValueNames As Variant, arrTypes As Variant, uValue As Variant
        Dim strText As String, strLocation As String, strDescription As String
              
        ' Get Access version, computer name, registry tree and key path
        strAccessVersion = SysCmd(acSysCmdAccessVer)
        
        strComputer = "." ' Use . for current machine
        hDefKey = HKEY_CURRENT_USER

       ' Connect to registry provider on target machine with current user
        Set oReg = GetObject("winmgmts:{impersonationLevel=impersonate}!\\" & strComputer & "\root\default:StdRegProv")

        strKeyPath = "SOFTWARE\Microsoft\Office\" & strAccessVersion & "\Access\Security"

        ' Enum the keys of the key path we've chosen (if any)
        oReg.EnumKey hDefKey, strKeyPath, arrSubKeys

        For Each strSubkey In arrSubKeys
          'First check in the top level folder for a AllLocationDisabled key
          ' Get the subkey value names and types
          strSubKeyPath = strKeyPath & "\" & strSubkey
          oReg.EnumValues hDefKey, strSubKeyPath, arrValueNames, arrTypes

          For I = LBound(arrValueNames) To UBound(arrValueNames)
            strValueName = arrValueNames(I)

            oReg.GetDWORDValue hDefKey, strSubKeyPath, strValueName, uValue
                ' Debug.Print strSubKeyPath, strValueName, uValue

            'Check whether all trusted locations are disabled
               If strSubkey = "Trusted Locations" And strValueName = "AllLocationsDisabled" And uValue = 1 Then
                  MsgBox "All Trusted Locations have been disabled for Access " & strAccessVersion, vbCritical, "Trusted Locations Disabled"
                  Exit Function
              ' Else
                ' MsgBox "Key missing"
               End If
          Next I

        Next strSubkey

      '===========================================================

      'Now look at each Trusted Location if any exist
      strKeyPath = "SOFTWARE\Microsoft\Office\" & strAccessVersion & "\Access\Security\Trusted Locations"

      ' Enum the subkeys of the key path we've chosen
              oReg.EnumKey hDefKey, strKeyPath, arrSubKeys
'
              For Each strSubkey In arrSubKeys

                ' Get the subkey value names and types
                strSubKeyPath = strKeyPath & "\" & strSubkey
                oReg.EnumValues hDefKey, strSubKeyPath, arrValueNames, arrTypes

                For I = LBound(arrValueNames) To UBound(arrValueNames)

                  strValueName = arrValueNames(I)
                  'Debug.Print strSubkey, I, strValueName

                  Select Case arrTypes(I)

                    ' Check the REG_SZ values for Path
                    Case REG_SZ
                      oReg.GetStringValue hDefKey, strSubKeyPath, strValueName, strValue
                      
                      If strValueName = "Path" Then
                            If CurrentProject.Path & "\" <> strValue And InStr(CurrentProject.Path, strValue) <> 0 Then 'match found as subfolder of path
                                ' Debug.Print strSubKeyPath, strValue
                                strLocation = strValue
                              'Now look for Description & AllowSubfolders keys in that path
                                GoTo MoreInfo
    
                              ElseIf CurrentProject.Path & "\" = strValue Then 'match found as folder path
                                 ' Debug.Print strSubKeyPath, strValue
                                  strLocation = strValue
                                  IsTrustedLocation = True
                                  
                                  'now get Description & AllowSubfolders keys if these exist
                                  GoTo MoreInfo
                              Else
                                  'no match - check next Trusted Location
                                  GoTo NextSubkey
                              End If
                      End If

MoreInfo:
                   'Get the Description if it exists
                      If strValueName = "Description" Then
                          strDescription = strValue
                      End If

                   ' Get the REG_DWORD value for AllowSubfolders if it exists
                    Case REG_DWORD
                          oReg.GetDWORDValue hDefKey, strSubKeyPath, strValueName, uValue
                          'Debug.Print strSubKeyPath, strValueName, uValue
                          If strValueName = "AllowSubfolders" And uValue = 1 Then
                              IsTrustedLocation = True
                          End If
                  End Select
                Next I

NextSubkey:
              If IsTrustedLocation = True Then GoTo Trusted
              Next strSubkey

Trusted:
'                    'build info for immediate window (if required)
                       Debug.Print "Current Project Folder = " & CurrentProject.Path
                       Debug.Print ""
                       Debug.Print "IsTrustedLocation = " & IsTrustedLocation

                          'additional info
                       If IsTrustedLocation Then
                          Debug.Print "The current project folder is trusted for Access " & strAccessVersion
                          Debug.Print "-----------------------------------------------------"
                          Debug.Print "TrustedLocation = " & strSubkey
                          Debug.Print "Path = " & strLocation
                          If strDescription <> "" Then Debug.Print "Description = " & strDescription
                          If uValue <> "" Then Debug.Print "AllowSubfolders = " & uValue & IIf(uValue = 1, "   (Yes)", "   (No)")
                       End If

        'build message box string (if required)
        strText = "IsTrustedLocation = " & IsTrustedLocation & vbCrLf & vbCrLf & "Current Project Folder = " & CurrentProject.Path & "            "

        'additional info
        If IsTrustedLocation Then
           strText = strText & "@The current project folder is trusted for Access " & strAccessVersion
           strText = strText & vbCrLf & "------------------------------------------------------"
           strText = strText & vbCrLf & "TrustedLocation: " & String(10, " ") & strSubkey
           strText = strText & vbCrLf & "Path: " & String(30, " ") & strLocation
           If strDescription <> "" Then strText = strText & vbCrLf & "Description: " & String(18, " ") & strDescription
           If uValue <> "" Then
                strText = strText & vbCrLf & "AllowSubfolders: " & String(9, " ") & uValue & IIf(uValue = 1, "   (Yes)", "   (No)") & "@"
           Else
                strText = strText & "@"
           End If
        End If

        FormattedMsgBox strText, vbInformation, "Trusted Location Info"

Exit_Handler:
    Exit Function

Err_Handler:
    If Err = 13 Then 'type mismatch
        If strSubkey = "Trusted Locations" Then 'AllLocationsDisabled key doesn't exist
            Err = 0: Resume Next 'ignore error & continue
        Else 'no trusted locations exist
            MsgBox "There are currently no trusted locations for Access " & strAccessVersion, vbExclamation, "No Trusted Locations"
        End If
    Else
        MsgBox "Error " & Err & " in IsTrustedLocation procedure: " & vbCrLf & Err.Description
    End If
    GoTo Exit_Handler

End Function

Public Function FormattedMsgBox(Prompt As String, Optional Buttons As VbMsgBoxStyle = vbOKOnly, _
      Optional title As String = vbNullString, Optional HelpFile As Variant, Optional Context As Variant) As VbMsgBoxResult

On Error GoTo Err_Handler

'Originally from http://www.trigeminal.com/usenet/usenet015.asp (now defunct)

       FormattedMsgBox = Eval("MsgBox(""" & Prompt & """, " & Buttons & ", """ & title & """)")

Exit_Handler:
   Exit Function

Err_Handler:
   MsgBox "Error " & Err.Number & " in FormattedMsgBox procedure : " & vbCrLf & "   - " & Err.Description
   Resume Exit_Handler

End Function


